@@ -24,6 +24,7 @@ abstract class CControllerPopupItemTest extends CController {
 	const ZBX_TEST_TYPE_ITEM = 0;
 	const ZBX_TEST_TYPE_ITEM_PROTOTYPE = 1;
 	const ZBX_TEST_TYPE_LLD = 2;
+	const ZBX_TEST_TYPE_LLD_PROTOTYPE = 3;
 
 	/**
 	 * Max-length of input fields that can contain resolved macro values. Used in views for input fields.
@@ -94,10 +95,8 @@ abstract class CControllerPopupItemTest extends CController {
 
 	/**
 	 * Item types with mandatory item key.
-	 *
-	 * @var array
 	 */
-	protected $item_types_has_key_mandatory = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL,
+	public static array $item_types_has_key_mandatory = [ITEM_TYPE_ZABBIX, ITEM_TYPE_SIMPLE, ITEM_TYPE_INTERNAL,
 		ITEM_TYPE_EXTERNAL, ITEM_TYPE_DB_MONITOR, ITEM_TYPE_HTTPAGENT, ITEM_TYPE_IPMI,
 		ITEM_TYPE_SSH, ITEM_TYPE_TELNET, ITEM_TYPE_JMX, ITEM_TYPE_CALCULATED
 	];
@@ -407,7 +406,8 @@ abstract class CControllerPopupItemTest extends CController {
 						$item_flags = [
 							self::ZBX_TEST_TYPE_ITEM => ZBX_FLAG_DISCOVERY_NORMAL,
 							self::ZBX_TEST_TYPE_ITEM_PROTOTYPE => ZBX_FLAG_DISCOVERY_PROTOTYPE,
-							self::ZBX_TEST_TYPE_LLD => ZBX_FLAG_DISCOVERY_RULE
+							self::ZBX_TEST_TYPE_LLD => ZBX_FLAG_DISCOVERY_RULE,
+							self::ZBX_TEST_TYPE_LLD_PROTOTYPE => ZBX_FLAG_DISCOVERY_RULE_PROTOTYPE
 						];
 						$item_flag = $item_flags[$this->getInput('test_type')];
 					}
@@ -785,7 +785,9 @@ abstract class CControllerPopupItemTest extends CController {
 				'supported_macros' => array_diff_key($this->macros_by_item_props['key'],
 					['support_user_macros' => true, 'support_lld_macros' => true]
 				),
-				'support_lldmacros' => ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE),
+				'support_lldmacros' => in_array($this->test_type,
+					[self::ZBX_TEST_TYPE_ITEM_PROTOTYPE, self::ZBX_TEST_TYPE_LLD_PROTOTYPE]
+				),
 				'texts_support_macros' => [$inputs['key']],
 				'texts_support_lld_macros' => [$inputs['key']],
 				'texts_support_user_macros' => [$inputs['key']],
@@ -810,8 +812,12 @@ abstract class CControllerPopupItemTest extends CController {
 	 */
 	protected function resolvePreprocessingStepMacros(array $steps) {
 		// Resolve macros used in parameter fields.
-		$macros_posted = $this->getInput('macros', []);
-		$macros_types = ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE)
+		$macros_posted = $this->hasInput('macros')
+			? json_decode($this->getInput('macros'), true)
+			: [];
+		$macros_types = in_array($this->test_type,
+			[self::ZBX_TEST_TYPE_ITEM_PROTOTYPE, self::ZBX_TEST_TYPE_LLD_PROTOTYPE]
+		)
 			? ['usermacros' => true, 'lldmacros' => true]
 			: ['usermacros' => true];
 
@@ -855,7 +861,9 @@ abstract class CControllerPopupItemTest extends CController {
 
 		$expression_parser = new CExpressionParser([
 			'usermacros' => true,
-			'lldmacros' => ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE),
+			'lldmacros' => in_array($this->test_type,
+				[self::ZBX_TEST_TYPE_ITEM_PROTOTYPE, self::ZBX_TEST_TYPE_LLD_PROTOTYPE]
+			),
 			'calculated' => true,
 			'host_macro' => true,
 			'empty_host' => true
@@ -972,7 +980,9 @@ abstract class CControllerPopupItemTest extends CController {
 	 */
 	protected function resolveItemPropertyMacros(array $inputs) {
 		// Resolve macros used in parameter fields.
-		$macros_posted = $this->getInput('macros', []);
+		$macros_posted = $this->hasInput('macros')
+			? json_decode($this->getInput('macros'), true)
+			: [];
 
 		foreach (array_keys($this->macros_by_item_props) as $field) {
 			if (!array_key_exists($field, $inputs)) {
@@ -991,7 +1001,7 @@ abstract class CControllerPopupItemTest extends CController {
 				'macros_n' => []
 			];
 
-			if ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE) {
+			if (in_array($this->test_type, [self::ZBX_TEST_TYPE_ITEM_PROTOTYPE, self::ZBX_TEST_TYPE_LLD_PROTOTYPE])) {
 				$types += ['lldmacros' => true];
 			}
 
