@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -33,8 +28,7 @@ else {
 	$data = $_REQUEST;
 }
 
-if (is_array($data) && array_key_exists('method', $data)
-		&& in_array($data['method'], ['message.settings', 'message.get', 'zabbix.status'])) {
+if (is_array($data) && array_key_exists('method', $data) && $data['method'] === 'zabbix.status') {
 	CWebUser::disableSessionExtension();
 }
 
@@ -66,21 +60,47 @@ switch ($data['method']) {
 		break;
 
 	case 'zabbix.status':
-		if (!CSessionHelper::has('serverCheckResult')
-				|| (CSessionHelper::get('serverCheckTime') + SERVER_CHECK_INTERVAL) <= time()) {
-			$zabbixServer = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT,
-				timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::CONNECT_TIMEOUT)),
-				timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::SOCKET_TIMEOUT)), 0
-			);
-			CSessionHelper::set('serverCheckResult', $zabbixServer->isRunning(CSessionHelper::getId()));
-			CSessionHelper::set('serverCheckTime', time());
+		global $ZBX_SERVER, $ZBX_SERVER_PORT;
+
+		$now = time();
+		$check_time = CSessionHelper::get('serverCheckTime') ?? 0;
+
+		if ($check_time + SERVER_CHECK_INTERVAL <= $now || $now < $check_time) {
+			$status = true;
+			$error_code = 0;
+
+			if ($ZBX_SERVER === null && $ZBX_SERVER_PORT === null) {
+				$status = false;
+			}
+			else {
+				$zabbix_server = new CZabbixServer($ZBX_SERVER, $ZBX_SERVER_PORT,
+					timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::CONNECT_TIMEOUT)),
+					timeUnitToSeconds(CSettingsHelper::get(CSettingsHelper::SOCKET_TIMEOUT)), 0
+				);
+
+				if (!$zabbix_server->isRunning() && !$zabbix_server->canConnect(CSessionHelper::getId())) {
+					$status = false;
+					$error_code = $zabbix_server->getErrorCode();
+				}
+			}
+
+			CSessionHelper::set('serverCheckResult', $status);
+			CSessionHelper::set('serverCheckResultErrorCode', $error_code);
+			CSessionHelper::set('serverCheckTime', $now);
+		}
+
+		$server_status = CSessionHelper::get('serverCheckResult');
+		$message = '';
+
+		if ($server_status === false) {
+			$message = (CSessionHelper::get('serverCheckResultErrorCode') === CZabbixServer::ERROR_CODE_TLS)
+				? _('Unable to connect to the Zabbix server due to TLS settings. Some functions are unavailable.')
+				: _('Zabbix server is not running: the information displayed may not be current.');
 		}
 
 		$result = [
-			'result' => (bool) CSessionHelper::get('serverCheckResult'),
-			'message' => CSessionHelper::get('serverCheckResult')
-				? ''
-				: _('Zabbix server is not running: the information displayed may not be current.')
+			'result' => $server_status,
+			'message' => $message
 		];
 		break;
 
@@ -114,15 +134,14 @@ switch ($data['method']) {
 			]);
 
 			foreach ($triggers as $trigger) {
-				$trigger['class_name'] = getSeverityStyle($trigger['priority']);
+				$trigger['class_name'] = CSeverityHelper::getStyle((int) $trigger['priority']);
 				$result[] = $trigger;
 			}
 		}
 		break;
 
 	/**
-	 * Create multi select data.
-	 * Supported objects: "applications", "hosts", "hostGroup", "templates", "triggers", "application_prototypes"
+	 * Create multiselect data.
 	 *
 	 * @param string $data['object_name']
 	 * @param string $data['search']
@@ -133,42 +152,31 @@ switch ($data['method']) {
 	case 'multiselect.get':
 		switch ($data['object_name']) {
 			case 'hostGroup':
-				$options = [
+				$db_groups = API::HostGroup()->get([
 					'output' => ['groupid', 'name'],
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
-					'real_hosts' => array_key_exists('real_hosts', $data) ? $data['real_hosts'] : null,
-					'with_items' => array_key_exists('with_items', $data) ? $data['with_items'] : null,
-					'with_httptests' => array_key_exists('with_httptests', $data) ? $data['with_httptests'] : null,
-					'with_hosts_and_templates' => array_key_exists('with_hosts_and_templates', $data)
-						? $data['with_hosts_and_templates']
-						: null,
-					'with_monitored_triggers' => array_key_exists('with_monitored_triggers', $data)
-						? $data['with_monitored_triggers']
-						: null,
-					'with_triggers' => array_key_exists('with_triggers', $data) ? $data['with_triggers'] : null,
-					'templated_hosts' => array_key_exists('templated_hosts', $data) ? $data['templated_hosts'] : null,
-					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
+					'with_hosts' => array_key_exists('with_hosts', $data),
+					'with_items' => array_key_exists('with_items', $data),
+					'with_httptests' => array_key_exists('with_httptests', $data),
+					'with_monitored_triggers' => array_key_exists('with_monitored_triggers', $data),
+					'with_triggers' => array_key_exists('with_triggers', $data),
+					'editable' => array_key_exists('editable', $data),
 					'limit' => array_key_exists('limit', $data) ? $data['limit'] : null,
 					'preservekeys' => true
-				];
-				$hostGroups = API::HostGroup()->get($options);
+				]);
 
-				if ($hostGroups) {
-					if (array_key_exists('enrich_parent_groups', $data)) {
-						$hostGroups = enrichParentGroups($hostGroups);
-					}
-
-					CArrayHelper::sort($hostGroups, [
-						['field' => 'name', 'order' => ZBX_SORT_UP]
-					]);
-
-					if (isset($data['limit'])) {
-						$hostGroups = array_slice($hostGroups, 0, $data['limit']);
-					}
-
-					$result = CArrayHelper::renameObjectsKeys($hostGroups, ['groupid' => 'id']);
+				if (array_key_exists('enrich_parent_groups', $data)) {
+					$db_groups = enrichParentGroups($db_groups);
 				}
+
+				CArrayHelper::sort($db_groups, [['field' => 'name', 'order' => ZBX_SORT_UP]]);
+
+				if (array_key_exists('limit', $data)) {
+					$db_groups = array_slice($db_groups, 0, $data['limit']);
+				}
+
+				$result = CArrayHelper::renameObjectsKeys($db_groups, ['groupid' => 'id']);
 				break;
 
 			case 'host_templates':
@@ -180,8 +188,9 @@ switch ($data['method']) {
 					'with_httptests' => array_key_exists('with_httptests', $data) ? $data['with_httptests'] : null,
 					'with_triggers' => array_key_exists('with_triggers', $data) ? $data['with_triggers'] : null,
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
 					'editable' => array_key_exists('editable', $data) ? $data['editable'] : false,
-					'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT)
+					'limit' => $limit
 				];
 
 				if ($data['object_name'] === 'host_templates') {
@@ -219,29 +228,37 @@ switch ($data['method']) {
 			case 'items':
 			case 'item_prototypes':
 				$options = [
-					'output' => ['itemid', 'hostid', 'name', 'key_'],
+					'output' => ['itemid'],
 					'selectHosts' => ['name'],
 					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
 					'templated' => array_key_exists('real_hosts', $data) ? false : null,
-					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
-					'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT)
+					'limit' => $limit
 				];
 
 				if ($data['object_name'] === 'item_prototypes') {
+					$options['output'][] = 'name';
+					$options['search'] = array_key_exists('search', $data) ? ['name' => $data['search']] : null;
+
 					$records = API::ItemPrototype()->get($options);
 				}
 				else {
-					if (array_key_exists('webitems', $data)) {
-						$options['webitems'] = $data['webitems'];
-					}
+					$resolve_macros = array_key_exists('resolve_macros', $data) && $data['resolve_macros'];
+					$name_field = $resolve_macros ? 'name_resolved' : 'name';
+
+					$options['output'][] = $name_field;
+					$options['search'] = array_key_exists('search', $data) ? [$name_field => $data['search']] : null;
+					$options['webitems'] = true;
 
 					$records = API::Item()->get($options);
+
+					if ($resolve_macros) {
+						$records = CArrayHelper::renameObjectsKeys($records, ['name_resolved' => 'name']);
+					}
 				}
 
 				if ($records) {
-					$records = CMacrosResolverHelper::resolveItemNames($records);
-					CArrayHelper::sort($records, ['name_expanded']);
+					CArrayHelper::sort($records, ['name']);
 
 					if (array_key_exists('limit', $data)) {
 						$records = array_slice($records, 0, $data['limit']);
@@ -250,7 +267,7 @@ switch ($data['method']) {
 					foreach ($records as $record) {
 						$result[] = [
 							'id' => $record['itemid'],
-							'name' => $record['name_expanded'],
+							'name' => $record['name'],
 							'prefix' => $record['hosts'][0]['name'].NAME_DELIMITER
 						];
 					}
@@ -261,15 +278,18 @@ switch ($data['method']) {
 			case 'graph_prototypes':
 				$options = [
 					'output' => ['graphid', 'name'],
-					'selectHosts' => ['name'],
+					'selectHosts' => ['hostid', 'name'],
 					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
 					'templated' => array_key_exists('real_hosts', $data) ? false : null,
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
-					'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT)
+					'limit' => $limit
 				];
 
 				if ($data['object_name'] === 'graph_prototypes') {
+					$options['selectDiscoveryRule'] = ['hostid'];
+					$options['selectDiscoveryRulePrototype'] = ['hostid'];
+
 					$records = API::GraphPrototype()->get($options);
 				}
 				else {
@@ -283,10 +303,19 @@ switch ($data['method']) {
 				}
 
 				foreach ($records as $record) {
+					if ($data['object_name'] === 'graphs') {
+						$host_name = $record['hosts'][0]['name'];
+					}
+					else {
+						$host_names = array_column($record['hosts'], 'name', 'hostid');
+						$parent_lld = $record['discoveryRule'] ?: $record['discoveryRulePrototype'];
+						$host_name = $host_names[$parent_lld['hostid']];
+					}
+
 					$result[] = [
 						'id' => $record['graphid'],
 						'name' => $record['name'],
-						'prefix' => $record['hosts'][0]['name'].NAME_DELIMITER
+						'prefix' => $host_name.NAME_DELIMITER
 					];
 				}
 				break;
@@ -312,73 +341,66 @@ switch ($data['method']) {
 				}
 				break;
 
+			case 'templateGroup':
+				$db_groups = API::TemplateGroup()->get([
+					'output' => ['groupid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
+					'with_templates' => array_key_exists('with_templates', $data),
+					'with_items' => array_key_exists('with_items', $data),
+					'with_httptests' => array_key_exists('with_httptests', $data),
+					'with_triggers' => array_key_exists('with_triggers', $data),
+					'editable' => array_key_exists('editable', $data),
+					'limit' => array_key_exists('limit', $data) ? $data['limit'] : null,
+					'preservekeys' => true
+				]);
+
+				if (array_key_exists('enrich_parent_groups', $data)) {
+					$db_groups = enrichParentTemplateGroups($db_groups);
+				}
+
+				CArrayHelper::sort($db_groups, [['field' => 'name', 'order' => ZBX_SORT_UP]]);
+
+				if (array_key_exists('limit', $data)) {
+					$db_groups = array_slice($db_groups, 0, $data['limit']);
+				}
+
+				$result = CArrayHelper::renameObjectsKeys($db_groups, ['groupid' => 'id']);
+				break;
+
 			case 'proxies':
 				$proxies = API::Proxy()->get([
-					'output' => ['proxyid', 'host'],
-					'search' => array_key_exists('search', $data) ? ['host' => $data['search']] : null,
+					'output' => ['proxyid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'limit' => $limit
 				]);
 
 				if ($proxies) {
-					CArrayHelper::sort($proxies, ['host']);
+					CArrayHelper::sort($proxies, ['name']);
 
 					if (isset($data['limit'])) {
 						$proxies = array_slice($proxies, 0, $data['limit']);
 					}
 
-					$result = CArrayHelper::renameObjectsKeys($proxies, ['proxyid' => 'id', 'host' => 'name']);
+					$result = CArrayHelper::renameObjectsKeys($proxies, ['proxyid' => 'id']);
 				}
 				break;
 
-			case 'applications':
-				$applications = API::Application()->get([
-					'output' => ['applicationid', 'name'],
-					'hostids' => zbx_toArray($data['hostid']),
-					'search' => isset($data['search']) ? ['name' => $data['search']] : null,
+			case 'proxy_groups':
+				$db_proxy_groups = API::ProxyGroup()->get([
+					'output' => ['proxy_groupid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'limit' => $limit
 				]);
 
-				if ($applications) {
-					CArrayHelper::sort($applications, [
-						['field' => 'name', 'order' => ZBX_SORT_UP]
-					]);
+				if ($db_proxy_groups) {
+					CArrayHelper::sort($db_proxy_groups, ['name']);
 
-					if (isset($data['limit'])) {
-						$applications = array_slice($applications, 0, $data['limit']);
+					if (array_key_exists('limit', $data)) {
+						$db_proxy_groups = array_slice($db_proxy_groups, 0, $data['limit']);
 					}
 
-					$result = CArrayHelper::renameObjectsKeys($applications, ['applicationid' => 'id']);
-				}
-				break;
-
-			case 'application_prototypes':
-				$discovery_rules = API::DiscoveryRule()->get([
-					'output' => [],
-					'selectApplicationPrototypes' => ['application_prototypeid', 'name'],
-					'itemids' => [$data['parent_discoveryid']],
-					'limitSelects' => $limit
-				]);
-
-				if ($discovery_rules) {
-					$discovery_rule = $discovery_rules[0];
-
-					if ($discovery_rule['applicationPrototypes']) {
-						foreach ($discovery_rule['applicationPrototypes'] as $application_prototype) {
-							if (array_key_exists('search', $data)
-									&& stripos($application_prototype['name'], $data['search']) !== false) {
-								$result[] = [
-									'id' => $application_prototype['application_prototypeid'],
-									'name' => $application_prototype['name']
-								];
-							}
-						}
-
-						CArrayHelper::sort($result, [['field' => 'name', 'order' => ZBX_SORT_UP]]);
-
-						if (array_key_exists('limit', $data)) {
-							$result = array_slice($result, 0, $data['limit']);
-						}
-					}
+					$result = CArrayHelper::renameObjectsKeys($db_proxy_groups, ['proxy_groupid' => 'id']);
 				}
 				break;
 
@@ -393,6 +415,7 @@ switch ($data['method']) {
 					'selectHosts' => $host_fields,
 					'editable' => isset($data['editable']) ? $data['editable'] : false,
 					'monitored' => isset($data['monitored']) ? $data['monitored'] : null,
+					'templated' => array_key_exists('templated', $data) ? $data['templated'] : null,
 					'search' => isset($data['search']) ? ['description' => $data['search']] : null,
 					'limit' => $limit
 				]);
@@ -437,22 +460,32 @@ switch ($data['method']) {
 				break;
 
 			case 'users':
-				$users = API::User()->get([
-					'output' => ['userid', 'alias', 'name', 'surname'],
+				$options = [
+					'output' => ['userid', 'username', 'name', 'surname'],
 					'search' => array_key_exists('search', $data)
 						? [
-							'alias' => $data['search'],
+							'username' => $data['search'],
 							'name' => $data['search'],
 							'surname' => $data['search']
 						]
 						: null,
 					'searchByAny' => true,
 					'limit' => $limit
-				]);
+				];
+
+				if (array_key_exists('exclude_provisioned', $data) && $data['exclude_provisioned']) {
+					$options['filter']['userdirectoryid'] = 0;
+				}
+
+				$users = API::User()->get($options);
+
+				if (array_key_exists('context', $data) && stripos('system', $data['search']) !== false) {
+					$users[] = ['userid' => '0', 'username' => 'System', 'name' => '', 'surname' => ''];
+				}
 
 				if ($users) {
 					CArrayHelper::sort($users, [
-						['field' => 'alias', 'order' => ZBX_SORT_UP]
+						['field' => 'username', 'order' => ZBX_SORT_UP]
 					]);
 
 					if (array_key_exists('limit', $data)) {
@@ -469,11 +502,17 @@ switch ($data['method']) {
 				break;
 
 			case 'usersGroups':
-				$groups = API::UserGroup()->get([
+				$options = [
 					'output' => ['usrgrpid', 'name'],
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
 					'limit' => $limit
-				]);
+				];
+
+				if (array_key_exists('group_status', $data)) {
+					$options['status'] = $data['group_status'];
+				}
+
+				$groups = API::UserGroup()->get($options);
 
 				if ($groups) {
 					CArrayHelper::sort($groups, [
@@ -489,10 +528,16 @@ switch ($data['method']) {
 				break;
 
 			case 'drules':
+				$filter = [];
+
+				if (array_key_exists('enabled_only', $data) && $data['enabled_only']) {
+					$filter['status'] = DRULE_STATUS_ACTIVE;
+				}
+
 				$drules = API::DRule()->get([
 					'output' => ['druleid', 'name'],
 					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
-					'filter' => ['status' => DRULE_STATUS_ACTIVE],
+					'filter' => $filter,
 					'limit' => $limit
 				]);
 
@@ -501,10 +546,6 @@ switch ($data['method']) {
 						['field' => 'name', 'order' => ZBX_SORT_UP]
 					]);
 
-					if (array_key_exists('limit', $data)) {
-						$applications = array_slice($drules, 0, $data['limit']);
-					}
-
 					$result = CArrayHelper::renameObjectsKeys($drules, ['druleid' => 'id']);
 				}
 				break;
@@ -545,42 +586,286 @@ switch ($data['method']) {
 					$result[] = ['id' => $api_method, 'name' => $api_method];
 				}
 				break;
+
+			case 'valuemap_names':
+				if (!array_key_exists('hostids', $data) || !array_key_exists('context', $data)) {
+					break;
+				}
+
+				$hostids = $data['hostids'];
+
+				if (array_key_exists('with_inherited', $data)) {
+					$hostids = CTemplateHelper::getParentTemplatesRecursive($hostids, $data['context']);
+				}
+
+				$result = API::ValueMap()->get([
+					'output' => ['valuemapid', 'name'],
+					'hostids' => $hostids,
+					'search' => ['name' => $data['search'] ? $data['search'] : null],
+					'limit' => $limit
+				]);
+				$result = array_column($result, null, 'name');
+				$result = CArrayHelper::renameObjectsKeys($result, ['valuemapid' => 'id']);
+				CArrayHelper::sort($result, ['name']);
+				break;
+
+			case 'valuemaps':
+			case 'template_valuemaps':
+				if (!array_key_exists('hostids', $data) || !array_key_exists('context', $data)) {
+					break;
+				}
+
+				if ($data['context'] === 'host') {
+					$hosts = API::Host()->get([
+						'output' => ['name'],
+						'hostids' => $data['hostids'],
+						'preservekeys' => true
+					]);
+				}
+				else {
+					$hosts = API::Template()->get([
+						'output' => ['name'],
+						'templateids' => $data['hostids'],
+						'preservekeys' => true
+					]);
+				}
+
+				$valuemaps = API::ValueMap()->get([
+					'output' => ['valuemapid', 'name', 'hostid'],
+					'hostids' => $data['hostids'],
+					'search' => ['name' => $data['search'] ? $data['search'] : null],
+					'limit' => $limit
+				]);
+
+				foreach ($valuemaps as &$valuemap) {
+					$valuemap['prefix'] = $hosts[$valuemap['hostid']]['name'].NAME_DELIMITER;
+					unset($valuemap['hostid']);
+				}
+				unset($valuemap);
+
+				$result = CArrayHelper::renameObjectsKeys($valuemaps, ['valuemapid' => 'id']);
+				CArrayHelper::sort($result, ['name']);
+				break;
+
+			case 'dashboard':
+				$dashboards = API::Dashboard()->get([
+					'output' => ['dashboardid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'limit' => $limit
+				]);
+
+				if ($dashboards) {
+					CArrayHelper::sort($dashboards, [['field' => 'name', 'order' => ZBX_SORT_UP]]);
+
+					if (array_key_exists('limit', $data)) {
+						$dashboards = array_slice($dashboards, 0, $data['limit']);
+					}
+
+					$result = CArrayHelper::renameObjectsKeys($dashboards, ['dashboardid' => 'id']);
+				}
+				break;
+
+			case 'services':
+				$services = API::Service()->get([
+					'output' => ['serviceid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'limit' => $limit
+				]);
+
+				if ($services) {
+					CArrayHelper::sort($services, [['field' => 'name', 'order' => ZBX_SORT_UP]]);
+
+					if (array_key_exists('limit', $data)) {
+						$services = array_slice($services, 0, $data['limit']);
+					}
+
+					$result = CArrayHelper::renameObjectsKeys($services, ['serviceid' => 'id']);
+				}
+				break;
+
+			case 'sla':
+				$slas = API::Sla()->get([
+					'output' => ['slaid', 'name'],
+					'filter' => [
+						'status' => array_key_exists('enabled_only', $data) ? ZBX_SLA_STATUS_ENABLED : null
+					],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'limit' => $limit
+				]);
+
+				if ($slas) {
+					CArrayHelper::sort($slas, [['field' => 'name', 'order' => ZBX_SORT_UP]]);
+
+					if (array_key_exists('limit', $data)) {
+						$slas = array_slice($slas, 0, $data['limit']);
+					}
+
+					$result = CArrayHelper::renameObjectsKeys($slas, ['slaid' => 'id']);
+				}
+				break;
+
+			case 'actions':
+				$actions = API::Action()->get([
+					'output' => ['actionid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'searchByAny' => true,
+					'limit' => $limit
+				]);
+
+				if ($actions) {
+					CArrayHelper::sort($actions, [
+						['field' => 'name', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$actions = array_slice($actions, 0, $data['limit']);
+					}
+
+					foreach ($actions as $action) {
+						$result[] = [
+							'id' => $action['actionid'],
+							'name' => $action['name']
+						];
+					}
+				}
+				break;
+
+			case 'media_types':
+				$media_types = API::MediaType()->get([
+					'output' => ['mediatypeid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'searchByAny' => true,
+					'limit' => $limit
+				]);
+
+				if ($media_types) {
+					CArrayHelper::sort($media_types, [
+						['field' => 'name', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$media_types = array_slice($media_types, 0, $data['limit']);
+					}
+
+					foreach ($media_types as $media_type) {
+						$result[] = [
+							'id' => $media_type['mediatypeid'],
+							'name' => $media_type['name']
+						];
+					}
+				}
+				break;
+
+			case 'sysmaps':
+				$sysmaps = API::Map()->get([
+					'output' => ['sysmapid', 'name'],
+					'search' => array_key_exists('search', $data) ? ['name' => $data['search']] : null,
+					'limit' => $limit
+				]);
+
+				if ($sysmaps) {
+					CArrayHelper::sort($sysmaps, [
+						['field' => 'name', 'order' => ZBX_SORT_UP]
+					]);
+
+					if (array_key_exists('limit', $data)) {
+						$sysmaps = array_slice($sysmaps, 0, $data['limit']);
+					}
+
+					$result = CArrayHelper::renameObjectsKeys($sysmaps, ['sysmapid' => 'id']);
+				}
+				break;
+
+			case 'host_inventory':
+				$inventory_fields = array_column(getHostInventories(true), 'title', 'nr');
+
+				if (array_key_exists('search', $data)) {
+					$inventory_fields = preg_grep('/'.preg_quote($data['search']).'/i', $inventory_fields);
+				}
+
+				foreach (array_slice($inventory_fields, 0, $limit, true) as $nr => $title) {
+					$result[] = ['id' => (string) $nr, 'name' => $title];
+				}
+				break;
 		}
 		break;
 
 	case 'patternselect.get':
 		$search = (array_key_exists('search', $data) && $data['search'] !== '') ? $data['search'] : null;
-		$wildcard_enabled = (strpos($search, '*') !== false);
-		$result = [];
+		$hostids = array_key_exists('hostids', $data) ? $data['hostids'] : null;
+		$items = array_key_exists('items', $data) ? $data['items'] : null;
+		$groupids = array_key_exists('groupids', $data) ? getSubGroups($data['groupids']) : null;
+		$wildcard_enabled = array_key_exists('wildcard_allowed', $data) && strpos($search, '*') !== false;
 
 		switch ($data['object_name']) {
 			case 'hosts':
 				$options = [
 					'output' => ['name'],
+					'groupids' => $groupids,
 					'search' => ['name' => $search.($wildcard_enabled ? '*' : '')],
 					'searchWildcardsEnabled' => $wildcard_enabled,
 					'preservekeys' => true,
-					'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT)
+					'sortfield' => 'name',
+					'limit' => $limit
 				];
 
 				$db_result = API::Host()->get($options);
 				break;
 
 			case 'items':
+				if (array_key_exists('host_pattern', $data)) {
+					$host_pattern_multiple = array_key_exists('host_pattern_multiple', $data)
+						&& $data['host_pattern_multiple'] == 1;
+					$host_patterns = $host_pattern_multiple ? $data['host_pattern'] : [$data['host_pattern']];
+					$host_pattern_wildcard_enabled = array_key_exists('host_pattern_wildcard_allowed', $data)
+						&& !in_array('*', $host_patterns, true);
+
+					$hosts = API::Host()->get([
+						'output' => [],
+						'hostids' => $hostids,
+						'groupids' => $groupids,
+						'search' => [
+							'name' => $host_pattern_wildcard_enabled ? $host_patterns : null
+						],
+						'searchWildcardsEnabled' => $host_pattern_wildcard_enabled,
+						'searchByAny' => true,
+						'preservekeys' => true
+					]);
+
+					$hostids = array_keys($hosts);
+				}
+
+				if (array_key_exists('hostid', $data)) {
+					$hostids = $data['hostid'];
+				}
+
+				$resolve_macros = array_key_exists('resolve_macros', $data) && $data['resolve_macros'];
+				$name_field = $resolve_macros ? 'name_resolved' : 'name';
+
 				$options = [
-					'output' => ['name'],
-					'search' => ['name' => $search.($wildcard_enabled ? '*' : '')],
+					'output' => ['itemid', $name_field],
+					'search' => [$name_field => $search.($wildcard_enabled ? '*' : '')],
 					'searchWildcardsEnabled' => $wildcard_enabled,
-					'filter' => [
-						'value_type' => [ITEM_VALUE_TYPE_UINT64, ITEM_VALUE_TYPE_FLOAT],
-						'flags' => ZBX_FLAG_DISCOVERY_NORMAL
-					],
+					'filter' => array_key_exists('filter', $data) ? $data['filter'] : null,
 					'templated' => array_key_exists('real_hosts', $data) ? false : null,
-					'webitems' => array_key_exists('webitems', $data) ? $data['webitems'] : null,
-					'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT)
+					'hostids' => $hostids,
+					'groupids' => $groupids,
+					'webitems' => true,
+					'sortfield' => 'name',
+					'limit' => $limit
 				];
 
+				if ($items) {
+					$options['search'][$name_field] = [...$items, $options['search'][$name_field]];
+					$options['searchWildcardsEnabled'] = true;
+					$options['searchByAny'] = true;
+				}
+
 				$db_result = API::Item()->get($options);
+
+				if ($resolve_macros) {
+					$db_result = CArrayHelper::renameObjectsKeys($db_result, ['name_resolved' => 'name']);
+				}
 				break;
 
 			case 'graphs':
@@ -590,7 +875,8 @@ switch ($data['method']) {
 					'hostids' => array_key_exists('hostid', $data) ? $data['hostid'] : null,
 					'templated' => array_key_exists('real_hosts', $data) ? false : null,
 					'searchWildcardsEnabled' => $wildcard_enabled,
-					'limit' => CSettingsHelper::get(CSettingsHelper::SEARCH_LIMIT)
+					'sortfield' => 'name',
+					'limit' => $limit
 				];
 
 				$db_result = API::Graph()->get($options);
@@ -603,7 +889,7 @@ switch ($data['method']) {
 		];
 
 		if ($db_result) {
-			$db_result = array_flip(zbx_objectValues($db_result, 'name'));
+			$db_result = array_flip(array_column($db_result, 'name'));
 
 			if (array_key_exists($search, $db_result)) {
 				unset($db_result[$search]);
@@ -622,6 +908,87 @@ switch ($data['method']) {
 		}
 		break;
 
+	case 'item_value_type.get':
+		$result = '';
+
+		if (array_key_exists('itemid', $data) && is_scalar($data['itemid'])) {
+			$items = API::Item()->get([
+				'output' => ['value_type'],
+				'itemids' => $data['itemid'],
+				'webitems' => true
+			]);
+
+			if ($items) {
+				$result = $items[0]['value_type'];
+			}
+		}
+		break;
+
+	case 'get_scripts_by_hosts':
+		$result = [];
+
+		if (array_key_exists('hostid', $data) && is_scalar($data['hostid'])) {
+			$scripts = API::Script()->getScriptsByHosts([
+				'hostid' => $data['hostid'],
+				'scriptid' => $data['scriptid'],
+				'manualinput' => $data['manualinput']
+			]);
+
+			$errors = CMessageHelper::getMessages();
+
+			if ($errors) {
+				$result = [
+					'error' => array_values(array_column($errors, 'message'))
+				];
+			}
+			elseif ($scripts) {
+				$result = $scripts[$data['hostid']][0];
+			}
+		}
+		break;
+
+	case 'get_scripts_by_events':
+		$result = [];
+
+		if (array_key_exists('eventid', $data) && is_scalar($data['eventid'])) {
+			$scripts = API::Script()->getScriptsByEvents([
+				'eventid' => $data['eventid'],
+				'scriptid' => $data['scriptid'],
+				'manualinput' => $data['manualinput']
+			]);
+
+			$errors = CMessageHelper::getMessages();
+
+			if ($errors) {
+				$result = [
+					'error' => array_values(array_column($errors, 'message'))
+				];
+			}
+			elseif ($scripts) {
+				$result = $scripts[$data['eventid']][0];
+			}
+		}
+		break;
+
+	case 'link_thresholds.validate':
+		$result = [];
+
+		if (array_key_exists('thresholds', $data)) {
+			$validation_rules = ['type' => API_OBJECTS, 'fields' => [
+				'threshold' => ['type' => API_NUMERIC]
+			]];
+
+			CApiInputValidator::validate($validation_rules, $data['thresholds'], 'thresholds', $error);
+
+			if ($error !== '') {
+				$result = [
+					'error' => $error
+				];
+			}
+		}
+
+		break;
+
 	default:
 		fatal_error('Wrong RPC call to JS RPC!');
 }
@@ -633,6 +1000,9 @@ if ($requestType == PAGE_TYPE_JSON) {
 			'result' => $result,
 			'id' => $data['id']
 		]);
+
+		session_write_close();
+		exit();
 	}
 }
 elseif ($requestType == PAGE_TYPE_TEXT_RETURN_JSON) {
@@ -640,6 +1010,9 @@ elseif ($requestType == PAGE_TYPE_TEXT_RETURN_JSON) {
 		'jsonrpc' => '2.0',
 		'result' => $result
 	]);
+
+	session_write_close();
+	exit();
 }
 elseif ($requestType == PAGE_TYPE_TEXT || $requestType == PAGE_TYPE_JS) {
 	echo $result;
